% Description: Vpoet odhadu BMD celej vstupnej matice (mxn), priemernch odhadov 
%              elementrnych zloiek pongizneho kostnho tkaniva a grafick zobrazenie 
%              vpotu sstavy rovnc.
% Input:    CT_numberH = obrazov matica (mxn, [double]) akvizcie CT dt o vyej energii
%           CT_numberL = obrazov matica (mxn, [double]) akvizcie CT dt o niej energii
%           z1 = seln hodnota [double] vyej energie
%           z2 = seln hodnota [double] niej energie
%           p = logick hodnota pozcie ROI pre grafick zobrazenie vpotu
%           sstavy rovnc [logical]
% Output:   BMD = odhad BMD pre kad pixel matice (mxn)
%           V_TB = odhad objemu matrix (kostn minerly+kolagn) [cm^3]  + figure grafickho zobrazenia vpotu sstavy rovnc
%           V_F = odhad objemu tukovho tkaniva [cm^3]                   + figure grafickho zobrazenia vpotu sstavy rovnc
%           V_T = odhad objemu tkaniva bez tuku [cm^3]
%           V_BM = odhad objemu minerlov [cm^3]
%           V_C = odhad objemu kolagnu [cm^3]
%           c_TBV = odhad celkovej hustoty pongiznej kosti [mg/cm^3]
%           c_Ca = odhad hustoty vpnika [mg/cm^3]
% -------------------------------------------------------------------
% Authors:  Martin Liska
%           Jiri Chmelik
% ===================================================================
function [ BMD, V_TB, V_F, V_T, V_BM, V_C, c_TBV, c_Ca ] = calculation( CT_numberH, CT_numberL, z1, z2, p )
%% DEFINCIA ENERGETICKCH KOEFICIENTOV PREMENCH (cell array)
CT_Coeff(:,1) = {'Photon Energy [keV]'; [40; 50; 60; 80; 100; 150]};
CT_Coeff(:,2) = {'\alpha (calcium hydroxyapatite)'; [3628; 2543; 1951; 1396; 1176; 993]};
CT_Coeff(:,3) = {'\eta (collagen)'; [902; 933; 950; 960; 963; 968]} ;
CT_Coeff(:,4) = {'\beta (adipose tissue)'; [894; 940; 966; 981; 990; 999]};
CT_Coeff(:,5) = {'\theta (red marrow)'; [938; 955; 971; 979; 983; 988]};

log_H = CT_Coeff{2,1} == z1 ;   % logick opercia pre indexciu hadanch koeficientov
log_L = CT_Coeff{2,1} == z2 ;   % logick opercia pre indexciu hadanch koeficientov
range = [0.001 0.8] ;             % rozpetie hodnot V_F (adipose tissue), defincia rozsahu x-osi v grafickom zobrazen

t = 0.92 ;                      % density of adipose tissue [g/cm^3]
D_TB = 1.92 ;                   % density of the matrix material (bone mineral+collagen) [g/cm^3]
g = 1.02 ;                      % density of nonadipose tissue [g/cm^3]
l = 3.06 ;                      % density of bone mineral alone [g/cm^3]
C = 1.38 ;                      % density of collagen alone [g/cm^3]
delta = -1000 ;                 % [HU]
epsilon_L = 0 ;                 % water offset number (low energy)
epsilon_H = 0 ;                 % water offset number (high energy)

%% VPOET KONTANT (LAMBDA, MI)
lambda = (l - D_TB)/(D_TB - C);                                                        % kontanta lambda
mi_L = ((CT_Coeff{2,2}(log_L)*l) + (CT_Coeff{2,3}(log_L)*C*lambda)) / (1 + lambda);    % kontanta mi (low energy)
mi_H = ((CT_Coeff{2,2}(log_H)*l) + (CT_Coeff{2,3}(log_H)*C*lambda)) / (1 + lambda);    % kontanta mi (high energy)

%% VPOET ODHADU BMD (LOW x HIGH ENERGY) PRE FAT x TOTAL BONE VOLUMES
    for i = 1:size(CT_numberL,1)
        for j = 1:size(CT_numberL,2)
            % SSTAVA ROVNC: A(1)*V_TB + A(2)*V_F = B
            A = [mi_L-(CT_Coeff{2,5}(log_L)*g), (CT_Coeff{2,4}(log_L)*t)-(CT_Coeff{2,5}(log_L)*g);...   % A matica (2x2): koeficienty pre V_TB a V_F...
                 mi_H-(CT_Coeff{2,5}(log_H)*g), (CT_Coeff{2,4}(log_H)*t)-(CT_Coeff{2,5}(log_H)*g)] ;    % [A(low)*V_TB, A(low)*V_F; A(high)*V_TB, A(high)*V_F]
            
            B = [CT_numberL(i,j)-(CT_Coeff{2,5}(log_L)*g)-delta-epsilon_L;...                           % B matica (1x2): hodnota sumy znmych premennch [low, high]
                 CT_numberH(i,j)-(CT_Coeff{2,5}(log_H)*g)-delta-epsilon_H] ;
            
            X = linsolve(A,B) ;                                                                         % X matica (1x2): [X1 X2] = [V_TB V_F], vpoet pomeru V_TB a V_F     
            
            V_TB(i,j) = X(1) ;                                                                                  % total bone volume [cm^3]
            V_F(i,j) = X(2) ;                                                                                   % adipose tissue volume [cm^3]
            V_T(i,j) = 1 - V_TB(i,j) - V_F(i,j) ;                                                               % non-adipose tissue volume [cm^3]
            V_BM(i,j) = V_TB(i,j) / (1+lambda) ;                                                                % BM volume [cm^3]
            V_C(i,j) = (lambda*V_TB(i,j)) / (1+lambda) ;                                                        % collagen volume [cm^3]
            BMD(i,j) = ((l * V_TB(i,j)) / (1 + lambda)) ;                                                       % BMD [mg/cm^3]
            c_TBV(i,j) = (BMD(i,j) + (C*lambda*V_TB(i,j)) / (1 + lambda) + (t*V_F(i,j)) + (g*V_T(i,j)))*1000 ;  % total density of trabecular space [mg/cm^3]
            BMD(i,j) = BMD(i,j) * 1000 ;                                                                        % BMD [mg/cm^3]
            c_Ca(i,j) = BMD(i,j) * 0.399 ;                                                                      % calcium concentration [mg/cm^3]
        end
    end
    
% vpoet priemrnch hodnt ROI elementrnych zloiek
V_TB = mean(V_TB(p));                                                       % objem kostnch minerlov a kolagnu [cm^3]
V_F = mean(V_F(p));                                                         % objem tukovho tkaniva [cm^3]
V_T = mean(V_T(p));                                                         % objem tkaniva bez tuku [cm^3]
V_BM = mean(V_BM(p));                                                       % objem kostnch minerlov [cm^3]
V_C = mean(V_C(p));                                                         % objem kolagnu [cm^3]
c_TBV = mean(c_TBV(p));                                                     % celkov hustota kostnho tkaniva [mg/cm^3]
c_Ca = mean(c_Ca(p));                                                       % koncentrcia vpniku [mg/cm^3]
%% VPOET PRIAMOK (LOW x HIGH ENERGY) PRE FAT x TOTAL BONE VOLUMES (INPUT: p => maska ROI) PRE GRAFICK ZOBRAZENIE
V_TB_L1 = (mean(mean(CT_numberL(p)))-(((CT_Coeff{2,4}(log_L)*t)-(CT_Coeff{2,5}(log_L)*g))*range(1))...  % vpoet prvho objemu V_TB na 0.1% rovni V_F (low energy)
            -(CT_Coeff{2,5}(log_L)*g)-delta-epsilon_L)/(mi_L-(CT_Coeff{2,5}(log_L)*g));
V_TB_L2 = (mean(mean(CT_numberL(p)))-(((CT_Coeff{2,4}(log_L)*t)-(CT_Coeff{2,5}(log_L)*g))*range(2))...  % vpoet druhho objemu V_TB na 80% rovni V_F (low energy)
            -(CT_Coeff{2,5}(log_L)*g)-delta-epsilon_L)/(mi_L-(CT_Coeff{2,5}(log_L)*g));
        
V_TB_H1 = (mean(mean(CT_numberH(p)))-(((CT_Coeff{2,4}(log_H)*t)-(CT_Coeff{2,5}(log_H)*g))*range(1))...  % vpoet prvho objemu V_TB na 0.1% rovni V_F (high energy)
            -(CT_Coeff{2,5}(log_H)*g)-delta-epsilon_H)/(mi_H-(CT_Coeff{2,5}(log_H)*g));
V_TB_H2 = (mean(mean(CT_numberH(p)))-(((CT_Coeff{2,4}(log_H)*t)-(CT_Coeff{2,5}(log_H)*g))*range(2))...  % vpoet druhho objemu V_TB na 80% rovni V_F (high energy)
            -(CT_Coeff{2,5}(log_H)*g)-delta-epsilon_H)/(mi_H-(CT_Coeff{2,5}(log_H)*g));
%% FIGURE: grafick zobrazenie vpotu sstavy rovnc       
figure('Name', 'Grafick rieenie z DVOJ-ENERGETICKCH CT dt')
plot([range(1) range(2)]*100,[V_TB_L1 V_TB_L2]*100,'r--','DisplayName',['Low keV data (', num2str(z2), ' keV)']);
title('Grafick zobrazenie objemov Fat - Total Bone')
xlabel('Volume Fat [%]')
ylabel('Total Bone Volume [%]')
hold on
plot([range(1) range(2)]*100,[V_TB_H1 V_TB_H2]*100,'g','DisplayName',['High keV data (', num2str(z1), ' keV)']);
hold on

% vpoet a grafick lokalizcia priesenku priamok
p1 = polyfit([range(1) range(2)]*100,[V_TB_L1 V_TB_L2]*100,1);
p2 = polyfit([range(1) range(2)]*100,[V_TB_H1 V_TB_H2]*100,1);
V_TB_p = fzero(@(x) polyval(p1-p2,x),3);
V_F_p = polyval(p1,V_TB_p);
figure(1)
plot(V_TB_p,V_F_p,'b+','DisplayName','Intersection');
legend('show','Location','northwest')
end