% Description: Function for computation of BMD using one of the two implemented methods depending on
% availability of fat tissue
% Input:    img - input data in HU
%           HU_muscle - value for muscle
%           HU_fat - value for fat
%           not_enough_fat - info about presence of fat in analysed data
% Output:   img_korr - BMD value
% -------------------------------------------------------------------
% Authors:  Oldrich Kodym
%           Jakub Salplachta
%           Jiri Jan
% ===================================================================
function img_korr = get_BMD( img, HU_muscle, HU_fat, not_enough_fat )

if not_enough_fat==true % metoda 3
    img_korr=(img-(HU_muscle-40))*0.7264+0.0011; % the linear conversion is used so that method 3 coresponds
                                               % with the results of method 1 on this scanning protocol, see tay_rescale.m

else % metoda 1
    coefs=polyfit([HU_fat, HU_muscle], [-85.7, 27.1], 1);
    img_korr=coefs(1)*img+coefs(2);
end
end

